﻿namespace Ovik.ViewModel
{
    using System.Reflection;
    using Autofac;
    using GalaSoft.MvvmLight;
    using GalaSoft.MvvmLight.Command;
    using GalaSoft.MvvmLight.Messaging;
    using Ovik.Messaging;
    using Ovik.Views;

    /// <summary>
    /// This class contains properties that a View can data bind to.
    /// <para>
    /// Use the <strong>mvvminpc</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// <para>
    /// You can also use Blend to data bind with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm/getstarted
    /// </para>
    /// </summary>
    public class AboutViewModel : ViewModelBase
    {
        /// <summary>
        /// Initializes a new instance of the AboutViewModel class.
        /// </summary>
        public AboutViewModel()
        {
            ////if (IsInDesignMode)
            ////{
            ////    // Code runs in Blend --> create design time data.
            ////}
            ////else
            ////{
            ////    // Code runs "for real": Connect to service, etc...
            ////}

            Close = new RelayCommand(OnAboutClose);

            Messenger.Default.Register<DisplayAboutMessage>(this, OnAboutMessage);
        }

        private IAboutView _view;

        public void OnAboutMessage(DisplayAboutMessage message)
        {
            _view = Container.Instance.Resolve<IAboutView>();
            
            //if (message.Show)
            //{
                _view.ShowDialog();
            //}
            //else
            //{
            //    _view.Close();
            //}
        }

        public string Version
        {
            get { return "Version: " + Assembly.GetExecutingAssembly().GetName().Version.ToString(); }
        }

        public RelayCommand Close { get; private set; }

        public void OnAboutClose()
        {
            _view.Close();

            //Messenger.Default.Send<DisplayAboutMessage>(new DisplayAboutMessage() { Show = false });
        }

        ////public override void Cleanup()
        ////{
        ////    // Clean own resources if needed

        ////    base.Cleanup();
        ////}
    }
}