﻿namespace Ovik.ViewModel
{
    using GalaSoft.MvvmLight;
    using GalaSoft.MvvmLight.Messaging;
    using Ovik.Messaging;
    using System.Diagnostics;
    using System;
    using System.Threading;
    using System.Collections.Generic;
    using Ovik.Model;
    using System.IO;
    using GalaSoft.MvvmLight.Command;
    using System.Windows;

    /// <summary>
    /// This class contains properties that a View can data bind to.
    /// <para>
    /// Use the <strong>mvvminpc</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// <para>
    /// You can also use Blend to data bind with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm/getstarted
    /// </para>
    /// </summary>
    public class ConvertViewModel : ViewModelBase
    {
        /// <summary>
        /// Initializes a new instance of the ConvertViewModel class.
        /// </summary>
        public ConvertViewModel()
        {
            ////if (IsInDesignMode)
            ////{
            ////    // Code runs in Blend --> create design time data.
            ////}
            ////else
            ////{
            ////    // Code runs "for real": Connect to service, etc...
            ////}

            Progress = 10;

            Messenger.Default.Register<ConvertVideosMessage>(this, OnConvertVideos);
            Messenger.Default.Register<AbortConversionMessage>(this, OnAbort);

            ShowOutput = new RelayCommand(OnShowOutput);
        }

        private void OnShowOutput()
        {
            if (OutputVisibility == Visibility.Collapsed)
            {
                OutputVisibility = Visibility.Visible;
                OutputButtonText = "Hide Output Details";
            }
            else
            {
                OutputVisibility = Visibility.Collapsed;
                OutputButtonText = "Show Output Details";
            }
        }

        public RelayCommand ShowOutput { get; private set; }

        private void OnAbort(AbortConversionMessage message)
        {
            if (_process == null || _process.HasExited)
            {
                return;
            }

            _process.Kill();
        }

        Process _process;

        public void ConvertVideos(object input)
        {
            List<VideoItem> videos = input as List<VideoItem>;

            for(int i = 0; i < videos.Count; i++)
            {
                ConversionStatus = string.Format("Processing {0} of {1} videos.", i+1, videos.Count);

                var video = videos[i];

                //var processInfo = new ProcessStartInfo("ffmpeg2theora-0.26.exe");
                var processInfo = new ProcessStartInfo("ffmpeg.exe");

                var path = video.Path;
                var filename = Path.GetFileNameWithoutExtension(path);
                var folder = Path.GetDirectoryName(path);
                var targetPath = Path.Combine(folder, filename + ".webm");

                if (File.Exists(targetPath))
                {
                    // Show confirmation dialog
                }

                // -i = input
                // -y = yes, overwrite
                // -threads can't be 0 (default).
                processInfo.Arguments = string.Format("-i \"{0}\" -y -threads 4 \"{1}\"", path, targetPath);
                //processInfo.Arguments = "\"" + video.Path + "\"";
                processInfo.UseShellExecute = false;
                processInfo.ErrorDialog = false;
                processInfo.CreateNoWindow = true;
                processInfo.RedirectStandardError = true;

                //ffmpeg -i [video file to encode] -b [bitrate]k -aq 60 -threads 4 output.webm
                //ffmpeg -i 123.mp4 -b 450k -ar 22050 -ab 48k -threads 2 output.webm

                try
                {
                    _process = new Process();
                    _process.EnableRaisingEvents = true;
                    _process.StartInfo = processInfo;
                    _process.ErrorDataReceived += new DataReceivedEventHandler(process_ErrorDataReceived);
                    _process.Start();
                    _process.BeginErrorReadLine();
                    //_process.WaitForExit();
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
            }
        }

        private void OnConvertVideos(ConvertVideosMessage message)
        {
            //GalaSoft.MvvmLight.Threading.DispatcherHelper.UIDispatcher.Invoke

            //ParameterizedThreadStart start = new ParameterizedThreadStart(ConvertVideos);
            //Thread thread = new Thread(start);
            //thread.Start(message.Videos);

            ConvertVideos(message.Videos);
        }

        void process_ErrorDataReceived(object sender, DataReceivedEventArgs e)
        {
            ParseOutput(e.Data);

            Output += e.Data + Environment.NewLine;
        }

        private void ParseOutput(string line)
        {
            if (string.IsNullOrEmpty(line))
            {
                return;
            }

            if (line.StartsWith("video:")) // Finished!
            {
                ConversionStatus = "Completed.";
                Progress = 100;
            }

            if (line.StartsWith("frame="))
            {
                int startIndex = line.IndexOf("size=");
                var size = line.Substring(startIndex + 5, line.IndexOf("time=") - startIndex - 5);
                OutputSize = size.Trim();
            }
        }

        public int Progress { get; set; }

        private string _output;

        public string Output
        {
            get { return _output; }
            set {
                if (_output != value)
                {
                    _output = value;
                    RaisePropertyChanged("Output");
                }
            }
        }

        private string _outputSize;

        public string OutputSize
        {
            get { return _outputSize; }
            set
            {
                if (_outputSize != value)
                {
                    _outputSize = value;
                    RaisePropertyChanged("OutputSize");
                }
            }
        }

        private string _outputButtonText = "Hide Output Details";

        public string OutputButtonText
        {
            get { return _outputButtonText; }
            set
            {
                if (_outputButtonText != value)
                {
                    _outputButtonText = value;
                    RaisePropertyChanged("OutputButtonText");
                }
            }
        }

        private Visibility _outputVisibility = Visibility.Visible;

        public Visibility OutputVisibility
        {
            get { return _outputVisibility; }
            set
            {
                if (_outputVisibility != value)
                {
                    _outputVisibility = value;
                    RaisePropertyChanged("OutputVisibility");
                }
            }
        }

        private string _conversionStatus = "";

        public string ConversionStatus
        {
            get { return _conversionStatus; }
            set
            {
                if (_conversionStatus != value)
                {
                    _conversionStatus = value;
                    RaisePropertyChanged("ConversionStatus");
                }
            }
        }

        ////public override void Cleanup()
        ////{
        ////    // Clean own resources if needed

        ////    base.Cleanup();
        ////}
    }
}