﻿namespace Ovik.ViewModel
{
    using System.Collections.ObjectModel;
    using GalaSoft.MvvmLight;
    using GalaSoft.MvvmLight.Command;
    using GalaSoft.MvvmLight.Messaging;
    using Ovik.Messaging;
    using Ovik.Model;
    using System.Windows.Controls;
    using Ovik.Views;
    using System.Windows.Navigation;
    using System.Collections.Generic;

    /// <summary>
    /// This class contains properties that a View can data bind to.
    /// <para>
    /// Use the <strong>mvvminpc</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// <para>
    /// You can also use Blend to data bind with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm/getstarted
    /// </para>
    /// </summary>
    public class MainViewModel : ViewModelBase
    {
        private ObservableCollection<VideoItem> _videos;
        //public ObservableCollection<Control> _pages;
        public List<string> _views;

        public List<string> Views
        {
            get { return _views; }
        }

        public string Title
        {
            get { return "Ovik"; }
        }

        public ObservableCollection<VideoItem> Videos
        {
            get
            {
                return _videos;
            }
        }

        private bool _allowDrop = true;
        public bool AllowDrop
        {
            get { return _allowDrop; }
            set
            
            {
                if (_allowDrop != value)
                {
                    _allowDrop = value;
                    RaisePropertyChanged("AllowDrop");
                }
            }
        }

        public string ActiveView
        {
            get { return Views[ViewIndex]; }
        }

        //public ObservableCollection<Control> Pages
        //{
        //    get { return _pages; }
        //}

        public RelayCommand DisplayAbout { get; private set; }

        public RelayCommand Previous { get; private set; }

        public RelayCommand Next { get; private set; }

        private int _viewIndex = 0;

        public int ViewIndex
        {
            get { return _viewIndex; }
            set {
                
                _viewIndex = value;

                UpdateIndexState();

                RaisePropertyChanged("ViewIndex");
                RaisePropertyChanged("ActiveView");
            }
        }

        private void UpdateIndexState()
        {
            if (ViewIndex == ViewCount - 1)
            {
                NextName = "Cancel";
            }
            else if (ViewIndex == ViewCount - 2)
            {
                NextName = "Convert";
            }
            else
            {
                NextName = "Next";
            }
        }

        public int ViewCount { get { return Views.Count; } }

        /// <summary>
        /// Initializes a new instance of the MainViewModel class.
        /// </summary>
        public MainViewModel()
        {
            _videos = new ObservableCollection<VideoItem>();

            _views = new List<string>();

            //_pages = new ObservableCollection<Control>();

            DisplayAbout = new RelayCommand(OnDisplayAbout);

            Previous = new RelayCommand(OnPrevious, PreviousCanExecute);

            Next = new RelayCommand(OnNext, NextCanExecute);

            ////if (IsInDesignMode)
            ////{
            ////    // Code runs in Blend --> create design time data.
            ////}
            ////else
            ////{
            ////    // Code runs "for real": Connect to service, etc...
            ////}

            InitializeViews();
        }

        private void InitializeViews()
        {
            _views.Add("Views/VideoCollectionView.xaml");
            _views.Add("Views/OptionsView.xaml");
            _views.Add("Views/ConvertView.xaml");
        }

        public void OnNext()
        {
            // Ensure we don't overflow the index.
            if (ViewIndex == ViewCount - 1)
            {
                // The user has clicked Cancel, abort conversion.
                Messenger.Default.Send<AbortConversionMessage>(new AbortConversionMessage());
                _nextDisabled = true;
                return;
            }

            ViewIndex++;

            if (ViewIndex == ViewCount - 1)
            {
                ViewModelLocator locator = new ViewModelLocator();
                //locator.VideoCollection.Videos

                Messenger.Default.Send<ConvertVideosMessage>(new ConvertVideosMessage { Videos = new List<VideoItem>(locator.VideoCollection.Videos), Options = locator.Options.Options });
            }
        }

        public void OnPrevious()
        {
            _nextDisabled = false;
            ViewIndex--;
        }

        private bool _nextDisabled = false;

        public bool NextCanExecute()
        {
            if (_nextDisabled)
            {
                return false;
            }

            return (ViewIndex < ViewCount);
        }

        public bool PreviousCanExecute()
        {
            return (ViewIndex > 0);
        }

        public void OnDisplayAbout()
        {
            Messenger.Default.Send<DisplayAboutMessage>(new DisplayAboutMessage { Show = true });
        }

        private string _nextName = "Next";
        public string NextName
        {
            get
            {
                return _nextName;
            }
            set
            {
                if (value != _nextName)
                {
                    _nextName = value;
                    RaisePropertyChanged("NextName");
                }
            }
        }

        ////public override void Cleanup()
        ////{
        ////    // Clean own resources if needed

        ////    base.Cleanup();
        ////}


    }
}