﻿namespace Ovik.ViewModel
{
    using GalaSoft.MvvmLight;
    using GalaSoft.MvvmLight.Command;
    using Microsoft.Win32;
    using System;
    using System.Collections.ObjectModel;
    using Ovik.Model;
    using GalaSoft.MvvmLight.Messaging;
    using Ovik.Messaging;
    using System.Windows;

    /// <summary>
    /// This class contains properties that a View can data bind to.
    /// <para>
    /// Use the <strong>mvvminpc</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// <para>
    /// You can also use Blend to data bind with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm/getstarted
    /// </para>
    /// </summary>
    public class VideoCollectionViewModel : ViewModelBase
    {
        /// <summary>
        /// Initializes a new instance of the VideoCollectionViewModel class.
        /// </summary>
        public VideoCollectionViewModel()
        {
            ////if (IsInDesignMode)
            ////{
            ////    // Code runs in Blend --> create design time data.
            ////}
            ////else
            ////{
            ////    // Code runs "for real": Connect to service, etc...
            ////}

            PickFiles = new RelayCommand(OnPickFiles);

            ClearFiles = new RelayCommand(OnClearFiles, ClearFilesCanExecute);

            _videos = new ObservableCollection<VideoItem>();

            Messenger.Default.Register<FileDropMessage>(this, OnFileDrop);

            InformationText = "Drag video files to convert or click the pick file(s) button.";
            InformationVisibility = Visibility.Visible;
        }

        private bool ClearFilesCanExecute()
        {
            return (Videos.Count > 0);
        }

        private void OnFileDrop(FileDropMessage message)
        {
            foreach(string file in message.Files)
            {
                InformationVisibility = Visibility.Collapsed;
                Videos.Add(new VideoItem { Path = file });
            }
        }

        private void OnPickFiles()
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.CheckFileExists = true;
            dialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.MyVideos);
            dialog.Multiselect = true;
            dialog.Title = "Pick any video file(s) to convert";
            dialog.Filter = "Video Files |*.avi;*.wmv;*.mpg;*.mpeg;*.mp4;|All Files (*.*) |*.*";

            var result = dialog.ShowDialog();

            if (result.HasValue && result.Value)
            {
                foreach (string file in dialog.FileNames)
                {
                    InformationVisibility = Visibility.Collapsed;
                    Videos.Add(new Model.VideoItem { Path = file });
                }
            }
        }

        private void OnClearFiles()
        {
            Videos.Clear();
            InformationVisibility = Visibility.Visible;
        }

        private ObservableCollection<VideoItem> _videos;

        public ObservableCollection<VideoItem> Videos
        {
            get
            {
                return _videos;
            }
        }

        public RelayCommand PickFiles { get; private set; }

        public RelayCommand ClearFiles { get; private set; }

        public string InformationText { get; set; }

        private Visibility _informationVisibility;

        public Visibility InformationVisibility
        {
            get { return _informationVisibility; }
            set
            {
                if (_informationVisibility != value)
                {
                    _informationVisibility = value;
                    RaisePropertyChanged("InformationVisibility");
                }
            }
        }

        ////public override void Cleanup()
        ////{
        ////    // Clean own resources if needed

        ////    base.Cleanup();
        ////}
    }
}