﻿/*
  In App.xaml:
  <Application.Resources>
      <vm:ViewModelLocator xmlns:vm="clr-namespace:Ovik.ViewModel"
                                   x:Key="Locator" />
  </Application.Resources>
  
  In the View:
  DataContext="{Binding Source={StaticResource Locator}, Path=ViewModelName}"
  
  OR (WPF only):
  
  xmlns:vm="clr-namespace:Ovik.ViewModel"
  DataContext="{Binding Source={x:Static vm:ViewModelLocator.ViewModelNameStatic}}"
*/
namespace Ovik.ViewModel
{
    using Ovik.ViewModel;

    /// <summary>
    /// This class contains static references to all the view models in the
    /// application and provides an entry point for the bindings.
    /// <para>
    /// Use the <strong>mvvmlocatorproperty</strong> snippet to add ViewModels
    /// to this locator.
    /// </para>
    /// <para>
    /// In Silverlight and WPF, place the ViewModelLocator in the App.xaml resources:
    /// </para>
    /// <code>
    /// &lt;Application.Resources&gt;
    ///     &lt;vm:ViewModelLocator xmlns:vm="clr-namespace:Ovik.ViewModel"
    ///                                  x:Key="Locator" /&gt;
    /// &lt;/Application.Resources&gt;
    /// </code>
    /// <para>
    /// Then use:
    /// </para>
    /// <code>
    /// DataContext="{Binding Source={StaticResource Locator}, Path=ViewModelName}"
    /// </code>
    /// <para>
    /// You can also use Blend to do all this with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm/getstarted
    /// </para>
    /// <para>
    /// In <strong>*WPF only*</strong> (and if databinding in Blend is not relevant), you can delete
    /// the ViewModelName property and bind to the ViewModelNameStatic property instead:
    /// </para>
    /// <code>
    /// xmlns:vm="clr-namespace:Ovik.ViewModel"
    /// DataContext="{Binding Source={x:Static vm:ViewModelLocator.ViewModelNameStatic}}"
    /// </code>
    /// </summary>
    public class ViewModelLocator
    {
        private static MainViewModel _main;
        private static OptionsViewModel _options;
        private static VideoCollectionViewModel _videoCollection;
        private static AboutViewModel _about;
        private static ConvertViewModel _convert;

        /// <summary>
        /// Initializes a new instance of the ViewModelLocator class.
        /// </summary>
        public ViewModelLocator()
        {
            ////if (ViewModelBase.IsInDesignModeStatic)
            ////{
            ////    // Create design time view models
            ////}
            ////else
            ////{
            ////    // Create run time view models
            ////}
            CreateViewModels();
        }

        private void CreateViewModels()
        {
            if (_main == null)
            {
                _main = new MainViewModel();
            }

            if (_options == null)
            {
                _options = new OptionsViewModel();
            }

            if (_videoCollection == null)
            {
                _videoCollection = new VideoCollectionViewModel();
            }

            if (_about == null)
            {
                _about = new AboutViewModel();
            }

            if (_convert == null)
            {
                _convert = new ConvertViewModel();
            }
        }

        ///// <summary>
        ///// Gets the Main property.
        ///// </summary>
        //public static MainViewModel MainStatic
        //{
        //    get
        //    {
        //        if (_main == null)
        //        {
        //            CreateMain();
        //        }

        //        return _main;
        //    }
        //}

        /// <summary>
        /// Gets the Main property.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance",
            "CA1822:MarkMembersAsStatic",
            Justification = "This non-static member is needed for data binding purposes.")]
        public MainViewModel Main
        {
            get
            {
                return _main;
            }
        }

        public OptionsViewModel Options
        {
            get { return _options; }
        }

        public VideoCollectionViewModel VideoCollection
        {
            get { return _videoCollection; }
        }

        public AboutViewModel About
        {
            get { return _about; }
        }

        public ConvertViewModel Convert
        {
            get { return _convert; }
        }

        /// <summary>
        /// Cleans up all the resources.
        /// </summary>
        public static void Cleanup()
        {
            _main.Cleanup();
            _main = null;

            _options.Cleanup();
            _options = null;

            _videoCollection.Cleanup();
            _videoCollection = null;

            _about.Cleanup();
            _about = null;

            _convert.Cleanup();
            _convert = null;
        }
    }
}